/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * JList with modules. Handles changing the current active module
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class DCModuleList extends DCList {

    private int currentIndex = -1;
    
    /**
     * Initializes this component. Adds all modules to the list
     */
	public DCModuleList() {
        super();
        addModules();
        addListSelectionListener(new moduleSelectAction());
	}

    /**
     * Adds all the modules  to the list
     */
	private void addModules() {
		Vector vector = new Vector();

		JPanel panelSoftware = new JPanel(new FlowLayout(FlowLayout.CENTER));
		panelSoftware.add(ComponentFactory.getLabel(IconLibrary._icoSoftwareModule));
        panelSoftware.add(ComponentFactory.getLabel(Resources.getText(DCLabels.lblSoftware)));

        JPanel panelMusicFiles = new JPanel(new FlowLayout(FlowLayout.CENTER));
        panelMusicFiles.add(ComponentFactory.getLabel(IconLibrary._icoMusicModule));
        panelMusicFiles.add(ComponentFactory.getLabel(Resources.getText(DCLabels.lblMusicFiles)));      
        
        JPanel panelMovies = new JPanel(new FlowLayout(FlowLayout.CENTER));
        panelMovies.add(ComponentFactory.getLabel(IconLibrary._icoMovieModule));
        panelMovies.add(ComponentFactory.getLabel(Resources.getText(DCLabels.lblMovies)));          
        
        JPanel panelAudioCD = new JPanel(new FlowLayout(FlowLayout.CENTER));
        panelAudioCD.add(ComponentFactory.getLabel(IconLibrary._icoAudioCDModule));
        panelAudioCD.add(ComponentFactory.getLabel(Resources.getText(DCLabels.lblAudioCDs)));        
        
        JPanel panelBooks = new JPanel(new FlowLayout(FlowLayout.CENTER));
        panelBooks.add(ComponentFactory.getLabel(IconLibrary._icoBookModule));
        panelBooks.add(ComponentFactory.getLabel(Resources.getText(DCLabels.lblBooks)));        

        JPanel panelHeader = new JPanel(Layout.getGBL());
        
        vector.addElement(panelHeader);
		vector.addElement(panelSoftware);
        vector.addElement(panelMovies);
        vector.addElement(panelMusicFiles);
        vector.addElement(panelAudioCD);
        
        // hack to make the module indices correspond to the model indices
        // (due to the not added Audio Track module)
        JLabel label = new JLabel();
        label.setMaximumSize(new Dimension(50,0));
        label.setPreferredSize(new Dimension(50,0));
        
        vector.insertElementAt(label, 5);
        vector.insertElementAt(panelBooks, 6);
        
        //add
		this.setListData(vector);
	}
    
    /**
     * Changes the current active module to the selected module.
     */
    private void setActiveModule() {
        int module = getSelectedIndex();
        if (module != currentIndex) {
            currentIndex = module;
            if (module != 0) {
                DataCrow.mainFrame.changeModule(module);
            }
        }
    }
    
    class moduleSelectAction implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent arg0) {
            if (arg0.getValueIsAdjusting()) {
                setActiveModule();
            }
        }
    }
}