package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ItemForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

public abstract class QuickViewPane extends JPanel {
    
    public static final int _DIRECTION_HORIZONTAL = 0;
    public static final int _DIRECTION_VERTICAL = 1;    
    
    protected final DCModule module;
    
    protected DCObject dco;
    private Collection images = new ArrayList();

    private JTextPane descriptionPane;
    private final JPanel panelImages = new JPanel(false);
    private final JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
    private final PopupMenu popupMenu = new PopupMenu();
    
    //private final Color background = new Color(255,255,233);
    
    private final HTMLEditorKit kit = new HTMLEditorKit();
    private final HTMLDocument document = new HTMLDocument();
    
    public QuickViewPane(DCModule module) {
        this.module = module;
        setLayout(Layout.getGBL());
        
        setMinimumSize(new Dimension(230, 300));
        setMaximumSize(new Dimension(230, 2000));

        buildPanel();
    }    
    
    public void setObject(DCObject dco) {
        this.dco = dco;
        
        String html = "<html><body " + getBodyStyle() + ">\n" +
                        getDescriptionTable() +
                      "</body> </html>";
        
        setDescriptionHtml(html);
        descriptionPane.setCaretPosition(0);
        setImages();
    }
    
    public void clear() {
        this.dco = null;
        descriptionPane.setText("");
    }
    
    private void setImages() {
        int y = 0;
        panelImages.removeAll();
        for (Iterator iter = images.iterator(); iter.hasNext(); y++) {
            String base64 = (String) iter.next();
            
            PictureField picField = ComponentFactory.getPictureField(true, false, false);
            picField.setMaximumSize(new Dimension(150,150));
            picField.setMinimumSize(new Dimension(150,150));
            picField.setPreferredSize(new Dimension(150,150));
            picField.setValue(base64);
            picField.setBorder(ComponentFactory.getBorder());
            
            panelImages.add(picField, Layout.getGBC(0, y, 1, 1, 1.0, 1.0,
                            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(2, 2, 2, 2), 0, 0));
        }
        panelImages.repaint();
        panelImages.revalidate();
        images.clear();
    }
    
    protected abstract String getDescriptionTable();
    
    protected void setDescriptionHtml(String html) {
        try {
            descriptionPane.setText("");
            kit.insertHTML(document, 0, html, 0, 0, null);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    protected String addTableRow(String table, int index, int textDirection) {
        Font labelFont = DcSettings.getValueAsFont(Repository.settings.stSystemFontBold);
        Font textFont = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        String textStyle = getStyle(textFont);
        String labelStyle = getStyle(labelFont);

        boolean horizontal = textDirection == _DIRECTION_HORIZONTAL;
        
        Object value = dco.getDisplayString(index);
        String s = value == null ? "" : value.toString();
        if (s != null && s.length() > 0 && !s.equals("-1") && !s.equals("0")) {
            if (!horizontal) {
                table += "<tr><td " + labelStyle + ">\n";
            } else {
                table += "<tr><td " + textStyle + ">\n";
            }
                
            table += "<b>" + dco.getLabel(index) + "</b>";
            
            if (!horizontal) {
                table += "</td></tr>";
                table += "<tr><td " + textStyle + ">\n";
            } else {
                table += " ";
            }
            
            table += value;
            table += "</td></tr>";
            table += "<tr><td></td></tr>";
        } else if (dco.getField(index).getValueType() == Repository.datatypes._OBJECT) {
            String base64 = (String) dco.getValue(index);
            if (base64 != null && base64.length() > 0) 
                images.add(base64);
        }
        return table;
    }    
    
    private void buildPanel() {
        // description panel
        descriptionPane = ComponentFactory.getTextPane();
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);
        descriptionPane.setEnabled(false);
        descriptionPane.setBounds(1,1,1,1);
        descriptionPane.addMouseListener(new QuickViewMouseListener());

        JScrollPane scroller = new JScrollPane(descriptionPane);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setBorder(null);

        // images panel
        panelImages.setLayout(Layout.getGBL());
        panelImages.addMouseListener(new QuickViewMouseListener());
        
        // tabbed pane
        tabbedPane.addTab("Description", scroller);
        tabbedPane.addTab("Images", panelImages);
        
        add(tabbedPane, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(0, 0, 0, 0), 0, 0));
        
        
        String html = "<html><body " + getBodyStyle() + "></body> </html>";
        setDescriptionHtml(html);
    }    
    
    private void showPopupMenu(int x, int y) {
        popupMenu.show(this, x, y);
    }    
    
    protected String getStyle(Font font) {
        int fontsize = font.getSize() > 10 ? font.getSize() : 11;
        
        String style = "style=" + '"' + "font-family:" + font.getFamily() + ";" + 
                       "font-size:" + fontsize + ";" + '"';
        return style;
    }
    
    protected String getBodyStyle() {
        
        Color colorEven = DcSettings.getValueAsColor(Repository.settings.stEvenRowColor);
        Color colorOdd = DcSettings.getValueAsColor(Repository.settings.stOddRowColor);
        
        String background = "";
        String foreground = "";
        if (colorEven.getRGB() < colorOdd.getRGB()) {
            background = Utilities.getHexColor(colorOdd);
            foreground = Utilities.getHexColor(colorOdd.darker().darker());
            descriptionPane.setBackground(colorOdd);
        } else {
            background = Utilities.getHexColor(colorEven);
            foreground = Utilities.getHexColor(colorEven.darker().darker());
            descriptionPane.setBackground(colorEven);
        }
        
        StringBuffer sb = new StringBuffer();
        sb.append("style=");
        sb.append('"');
        sb.append("background:;");
        String style = "style=width:205;" + '"' + "background:" + background + 
                       ";color:" + foreground + ";" + '"';
        return style;
    }

    public class PopupMenu extends JPopupMenu {
        public PopupMenu() {
            JMenuItem menuToggleLabels = new JMenuItem(Resources.getText(DCLabels.lblToggleQuickView));
            menuToggleLabels.addActionListener(new ToggleQuickViewAction());
            this.add(menuToggleLabels);
        }
    }    
    
    private class QuickViewMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                showPopupMenu(e.getX(), e.getY());
            } 

            if (e.getClickCount() == 2 && dco != null) {
                ItemForm itemform = new ItemForm(null, false, true, dco, module.isImdbUpdateAllowed(), module.isAmazonUpdateAllowed(), module.getSearchPanel().getClass(), dco.getRow());
                itemform.setVisible(true);
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}