/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.TableColumn;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCDialog;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.90
 */
public class FieldSelectorDialog extends DCDialog {
    
    private final JList leftList;
    private final JList rightList;
    private final JButton buttonLeft;
    private final JButton buttonRight;
    private final JButton buttonCancel;
    private final JButton buttonOK;
    
    private final TableColumn[] columns;
    
    private static boolean success = false;
    
    public FieldSelectorDialog(Collection visible, Collection hidden) {
    	super(DataCrow.mainFrame);
        setHelpIndex("dc.Tools.FieldSelector");
        setTitle(Resources.getText(DCLabels.lblFieldSelector));
        setBackground(DcSettings.getValueAsColor(Repository.settings.stColor));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setSizeSetting();
            }
        });
        
        leftList = ComponentFactory.getList(visible);
        rightList = ComponentFactory.getList(hidden);
        buttonRight = ComponentFactory.getButton(IconLibrary._icoNavigateRight);
        buttonLeft = ComponentFactory.getButton(IconLibrary._icoNavigateLeft);
        buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
        buttonOK = ComponentFactory.getButton(Resources.getText(DCLabels.lblOK));
        
        this.columns = new TableColumn[visible.size() + hidden.size()];
        
        int counter = 0;
        for (Iterator iter = visible.iterator(); iter.hasNext(); ) {
            columns[counter++] = (TableColumn) iter.next();
        }

        for (Iterator iter = hidden.iterator(); iter.hasNext();) {
            columns[counter++] = (TableColumn) iter.next();
        }

        buildDialog();
        init(visible, hidden);
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }
    
    /**
     * Initializes the selector with the correct hidden and visible fields
     * @param visible visible fields (table columns)
     * @param hidden hidden fields (table columns)
     */
    public void init(Collection visible, Collection hidden) {
        String[] headersVisible = new String[visible.size()];
        String[] headersHidden = new String[hidden.size()];
        
        int counter = 0;
        for (Iterator iter = visible.iterator(); iter.hasNext(); ) {
            headersVisible[counter++] = ((TableColumn) iter.next()).getHeaderValue().toString();   
        }
        
        counter = 0;
        for (Iterator iter = hidden.iterator(); iter.hasNext(); ) {
            headersHidden[counter++] = ((TableColumn) iter.next()).getHeaderValue().toString();   
        }
        
        leftList.setListData(headersVisible);
        rightList.setListData(headersHidden);
    }

    public static boolean isSuccessful() {
        return success;   
    }

    /**
     * Returns all visible fields
     * @return vector of Table Columns
     */
    public Collection getVisibleFields() {
        ArrayList alVisible = new ArrayList();
        for (int i = 0; i < leftList.getModel().getSize(); i++) {
            Object header = leftList.getModel().getElementAt(i);
            for (int j = 0; j < columns.length; j++) {
                if (header.equals(columns[j].getHeaderValue())) {
                	alVisible.add(columns[j]);
                }   
            }
        }
        return alVisible;
    }
    
    /**
     * Returns all hidden fields
     * @return vector of Table Columns
     */    
    public Collection getHiddenFields() {
        ArrayList fields = new ArrayList();
        for (int i = 0; i < rightList.getModel().getSize(); i++) {
            Object header = rightList.getModel().getElementAt(i);
            
            for (int j = 0; j < columns.length; j++) {
                if (header.equals(columns[j].getHeaderValue())) {
                    fields.add(columns[j]);
                }   
            }
        }
        return fields;
    }
    
    public void buildDialog() {
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        this.getContentPane().setBackground(color);

        JPanel panelLeft = new JPanel(false);
        panelLeft.setBackground(color);
        JPanel panelAction = new JPanel(false);
        panelAction.setBackground(color);
        JPanel panelRight = new JPanel(false);
        panelRight.setBackground(color);
        JPanel panelConfirm = new JPanel(false);
        panelConfirm.setBackground(color);
            
        //**********************************************************
        //Left panel
        //**********************************************************
        panelLeft.setLayout(Layout.getGBL());        
        JScrollPane scrollLeft = new JScrollPane(leftList);
        panelLeft.add(scrollLeft,     Layout.getGBC( 0, 0, 1, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
    
        //**********************************************************
        //Action panel
        //**********************************************************
        panelAction.setLayout(Layout.getGBL());    
        buttonRight.addActionListener(new MoveToRightAction());
        buttonLeft.addActionListener(new MoveToLeftAction()); 
        
        panelAction.add(buttonRight,  Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
        panelAction.add(buttonLeft,   Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
    
        //**********************************************************
        //Right panel
        //**********************************************************
        panelRight.setLayout(Layout.getGBL());        
        JScrollPane scrollRight = new JScrollPane(rightList);
        
        panelRight.add(scrollRight,   Layout.getGBC( 0, 0, 1, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                                      new Insets(5, 5, 5, 5), 0, 0));                         

        //**********************************************************
        //Confirmation panel
        //**********************************************************
        panelConfirm.setLayout(Layout.getGBL());  
        buttonOK.addActionListener(new OkAction());
        buttonCancel.addActionListener(new CancelAction());
        panelConfirm.add(buttonOK,      Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));                         
        panelConfirm.add(buttonCancel,  Layout.getGBC( 1, 0, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));  
    
        //**********************************************************
        //Main panel
        //**********************************************************            
        panelLeft.setBorder(    ComponentFactory.getTitleBorder(
                                Resources.getText(DCLabels.lblVisibleFields)));
        panelRight.setBorder(   ComponentFactory.getTitleBorder(
                                Resources.getText(DCLabels.lblHiddenFields)));
                                      
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(panelLeft,   Layout.getGBC( 0, 0, 1, 1, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(panelAction, Layout.getGBC( 1, 0, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(20, 10, 10, 10), 0, 0)); 
        this.getContentPane().add(panelRight,  Layout.getGBC( 2, 0, 1, 1, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(panelConfirm,Layout.getGBC( 0, 1, 3, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));                               
            
        this.setResizable(true);
        this.pack(); 
        this.setModal(true);
        this.setSize(DcSettings.getValueAsDimension(Repository.settings.stFieldSelectorSize));
    }    

    private void setSizeSetting() {
        DcSettings.setValue(Repository.settings.stFieldSelectorSize, getSize());
    }
    
    private class OkAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setSizeSetting();
            success = true;
            dispose();
        }        
    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            success = false;
            dispose();
        }        
    }
    
    private class MoveToRightAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int[] selected = leftList.getSelectedIndices();
            if (selected != null || selected.length > 0) {
                Vector vLeft = new Vector();
                Vector vRight = new Vector();
                for (int i = 0; i < leftList.getModel().getSize(); i++) {
                    boolean moved = false;
                    for (int j = 0; j < selected.length; j++) {
                        if (i == selected[j]) {
                            vRight.addElement(leftList.getModel().getElementAt(i));
                            moved = true;
                        }   
                    }
                    
                    if (!moved) {
                        vLeft.addElement(leftList.getModel().getElementAt(i));
                    }                        
                }
                
                for (int i = 0; i < rightList.getModel().getSize(); i++) {
                    vRight.addElement(rightList.getModel().getElementAt(i));
                }
                
                leftList.setListData(vLeft);
                rightList.setListData(vRight);
            }
        }
    }
    
    private class MoveToLeftAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int[] selected = rightList.getSelectedIndices();
            if (selected != null || selected.length > 0) {
                Vector vLeft = new Vector();
                Vector vRight = new Vector();
                for (int i = 0; i < rightList.getModel().getSize(); i++) {
                    boolean moved = false;
                    for (int j = 0; j < selected.length; j++) {
                        if (i == selected[j]) {
                            vLeft.addElement(rightList.getModel().getElementAt(i));
                            moved = true;
                        }   
                    }
                    
                    if (!moved) {
                        vRight.addElement(rightList.getModel().getElementAt(i));
                    }                        
                }
                
                for (int i = 0; i < leftList.getModel().getSize(); i++) {
                    vLeft.addElement(leftList.getModel().getElementAt(i));
                }
                
                leftList.setListData(vLeft);
                rightList.setListData(vRight);
            }
        }
    }
}
