/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * The expert dialog shows system information and allows the user
 * to connect directly to the database. There also some system tools
 * available via this dialog 
 *  
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.6
 */
public class ExpertForm extends DCFrame {

    /**
     * Builds this form and sets the help index
     */
    public ExpertForm(JFrame parent) {
        super(parent);
        
        buildForm();
        
        setHelpIndex("dc.Tools.ExpertUser");
        
        Image img = IconLibrary._icoMenuOption.getImage(); 
        setIconImage(img);
        img.flush();
        
        setTitle(Resources.getText(DCLabels.lblExpertUser));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                finish();
            }
        });
    }
    
    /**
     * Disposes this form and saves the settings
     */
    private void finish() {
        DcSettings.setValue(Repository.settings.stExpertFormSize, getSize());
        dispose();
    }
    
    private void buildForm() {
        Color background = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        JTabbedPane tabbedPane = new JTabbedPane();
        
    	SystemInfoPanel systemInfoPanel = new SystemInfoPanel();
        QueryPanel queryPanel = new QueryPanel();
        MaintenancePanel maintenancePanel = new MaintenancePanel();
        
        JPanel systemPanel = new JPanel();
        systemPanel.setLayout(Layout.getGBL());
        
        systemInfoPanel.setBorder(ComponentFactory.getTitleBorder(
                Resources.getText(DCLabels.lblSystemInformation)));
        maintenancePanel.setBorder(ComponentFactory.getTitleBorder(
                Resources.getText(DCLabels.lblMaintenance)));
        
        systemPanel.add(systemInfoPanel,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 5, 5), 0, 0));
        systemPanel.add(maintenancePanel, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 5, 5), 0, 0));

        tabbedPane.addTab(Resources.getText(DCLabels.lblSystem),
                          IconLibrary._icoMenuOption,
                          systemPanel);
        tabbedPane.addTab(Resources.getText(DCLabels.lblSQLTool),
                          IconLibrary._icoSqlManager,
                          queryPanel);

        getContentPane().setLayout(Layout.getGBL());
        getContentPane().add(tabbedPane, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
        		            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
							 new Insets(0, 0, 0, 0), 0, 0));
        
        setBackground(background);
        
        pack();
        
        setSize(DcSettings.getValueAsDimension(Repository.settings.stExpertFormSize));
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }
}
