/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import net.sf.dc.console.DCPanel;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCTooltips;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.Resources;

/**
 * The maintenance panel contains system and maintenance tools
 *  
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.6
 */
public class MaintenancePanel extends DCPanel {

    public MaintenancePanel() {
        super(null, null);
        buildPanel();
    }
    
    private void buildPanel() {
        setLayout(Layout.getGBL());
        
        JButton buttonCompact = ComponentFactory.getButton(
                Resources.getText(DCLabels.lblCompactAndShutdown));
        buttonCompact.setToolTipText(
                Resources.getText(DCTooltips.tpCompactAndShutdown));
        buttonCompact.addActionListener(new CompactAndShutdown());
        
        add(buttonCompact);
    }
    
    /**
     * Executes the shutdown and compact command on the database.
     * The database files are rewritten to make sure the minimum
     * amount of space is used
     */
    private void compactAndShutDown() {
        WorkFlow wf = new WorkFlow();
        try {
        	wf.runQueryDirectUnclosed("SHUTDOWN COMPACT;");
            DataCrow.mainFrame.finish(true);
        } catch (Exception exp) {
        	new MessageBox(exp.getMessage(), MessageBox.ERROR);
        }
    }
    
    private class CompactAndShutdown implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
        	compactAndShutDown();
        }
    }
}
