/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.software;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;

import net.sf.dc.console.TableInsertPanel;
import net.sf.dc.console.TableSearchPanel;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.IconCache;
import net.sf.dc.core.objects.cache.CacheManager;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;


/**
 * A form consisting out of a search software category tab and
 * an insert tab for software categories. Basically this is a 
 * small implementation of the MainFrame..
 * 
 * With this form, software categories are maintained. When
 * this form is closed, the Software Category Cache is refreshed.
 * 
 * @see net.sf.dc.core.objects.cache.CacheManager
 * @see net.sf.dc.core.objects.software.SoftwareCategory
 */
public class SoftwareCategoryForm extends DCFrame {
    private JTabbedPane tabbedPane = new JTabbedPane();

    public final CatagorySearchPanel searchCat;
    public final CatagoryInsertPanel insertCat;

    public SoftwareCategoryForm(JFrame parent) {
        super(parent);
        
        this.setTitle(Resources.getText(DCLabels.lblSoftwareCategories));
        Image image = IconLibrary._icoCategory.getImage();
        this.setIconImage(image);
        image.flush();
        
        searchCat = new CatagorySearchPanel();
        insertCat = new CatagoryInsertPanel();
        
        tabbedPane.addTab(searchCat.getTitle(), 
                          IconLibrary._icoExisting, 
                          searchCat);
        tabbedPane.addTab(insertCat.getTitle(), 
                          IconLibrary._icoNew, 
                          insertCat);        
        getContentPane().add(tabbedPane);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        this.setBackgrounds(DcSettings.getValueAsColor(Repository.settings.stColor));
        pack();
        
        this.setSize(DcSettings.getValueAsDimension(
                     Repository.settings.stSoftwareCatFormSize));
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }
    
    public void setVisible(boolean b) {
        DCModules.getModule(DCModules._SOFTWARECAT).getSearchForm().searchValues();
    	super.setVisible(b);
    }

    public CatagorySearchPanel getSearchPanel() {
        return searchCat;
    }

    public CatagoryInsertPanel getInsertPanel() {
        return insertCat;
    }

    public void setBackgrounds(Color color) {
        searchCat.setBackground(color);
        insertCat.setBackground(color);
    }
    
    private void close() {
        DcSettings.setValue(Repository.settings.stSoftwareCatFormSize, getSize());
        
        // update the cache as values might have been changed
        CacheManager.refresh(CacheManager._SOFTWARECATEGORY);
        IconCache.refresh();
        dispose();        
    }

    public class CatagorySearchPanel extends TableSearchPanel {
        public CatagorySearchPanel () {
            super(DCModules.getModule(DCModules._SOFTWARECAT),
                  Resources.getText(DCLabels.lblSoftwareCategory), 
                  IconLibrary._icoExisting);
        }
        
        public JComponent[] getExtraComponents() {
            JButton buttonRefresh = ComponentFactory.getButton(
                                    Resources.getText(DCLabels.lblRefresh));
            buttonRefresh.addActionListener(new RefreshListener());
            return new JComponent[] {buttonRefresh};
        }
        
        public void removeNewChild (String sID) {}
        
        private class RefreshListener implements ActionListener {
    		public void actionPerformed(ActionEvent arg0) {
                DCModules.getModule(DCModules._SOFTWARECAT)
                    .getSearchForm().searchValues();
    		}   
        }
    }

    public class CatagoryInsertPanel extends TableInsertPanel {
        
        public CatagoryInsertPanel() {
            super(DCModules.getModule(DCModules._SOFTWARECAT),
                  Resources.getText(DCLabels.lblNewItem, 
                  		            Resources.getText(DCLabels.lblSoftwareCategories)), 
                  IconLibrary._icoNew);
        }
        
        public JComponent[] getExtraComponents() {
            return new JComponent[0];
        }   
        
        public Collection getChildren(String parentID) {
            return new ArrayList();
        }
        
        public void saveNewValues() {
        	super.saveNewValues();
        }
    }
}
