package net.sf.dc.console.wizards;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.components.LongTextField;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.CloseFrameRequest;
import net.sf.dc.wf.requests.FillTableRequest;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

public class NewItemWizard extends DCFrame {
    
    private final LinkedList panels = new LinkedList();
    private int current = 0;
    
    private LongTextField textHelp = ComponentFactory.getLongTextField(8000);
    private JButton buttonNext = ComponentFactory.getButton("Next");
    private JButton buttonBack = ComponentFactory.getButton("Back");
    private JButton buttonFinish = ComponentFactory.getButton("Finish");
    
    private DCObject dco;
    
    public NewItemWizard(DCFrame parent) {
        super(parent);
        setIconImage(IconLibrary._icoMenuWizard16.getImage());
        
        int i = DcSettings.getValueAsInt(Repository.settings.stModule);
        DCModule module = DCModules.getModule(i);
        dco = module.getDCObject();
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                saveSettings();
            }
        });
        
        if (    module.isAmazonUpdateAllowed() || 
                module.isFreedBUpdateAllowed() || 
                module.isImdbUpdateAllowed()) {
            
            JPanel panelInternetSearch = new InternetWizardPanel(module, dco);
            JPanel panelName = new NameWizardPanel(dco);
            panels.add(panelName);
            panels.add(panelInternetSearch);
        }
        
        panels.add(new ItemDetailsWizardPanel(module, dco));
        build(panels);   
        
        setSize(DcSettings.getValueAsDimension(Repository.settings.stWizardFormSize));
        
        Dimension dim = Utilities.getCenteredWindowLocation(getSize());
        setLocation(dim.width, dim.height);
    }
    
    private void applyPanel() {
        buttonBack.setEnabled(current != 0);
        buttonNext.setVisible(current != panels.size() - 1);
        buttonFinish.setVisible(current == panels.size() - 1);

        textHelp.setText(getCurrent().getHelpText());
        
        String title = Resources.getText(DCMessages.msgNewItemWizard, 
                       new String[] {dco.getModule().getObjectName(), 
                                     String.valueOf(current + 1),
                                     String.valueOf(panels.size())});
        setTitle(title);
    }    
    
    private WizardPanel getCurrent() {
        return (WizardPanel) panels.get(current);
    }
    
    public void dispose() {
        saveSettings();
        super.dispose();
    }
    
    private void saveSettings() {
        DcSettings.setValue(Repository.settings.stWizardFormSize, getSize());
    }
    
    private void finish() {
        try {
            dco = getCurrent().apply();
            dco.addRequest(new CloseFrameRequest(this));
            dco.saveNew();
            searchSavedValues();
        } catch (Exception exp) {
            new MessageBox(exp.getMessage(), MessageBox._WARNING);
        }
    }
    
    private void searchSavedValues() {
        DCObject o =dco.getModule().getDCObject();
        o.setValue(DCObject._ID, dco.getID());
        
        o.addRequest(new FillTableRequest(dco.getModule().getIndex()));
        o.load(new QueryOptions(null, true, false));
        DataCrow.mainFrame.setSelectedTab(MainFrame._SEARCHTAB);        
    }
    
    private void next() {
        try {
            dco = getCurrent().apply();

            current += 1;
            if (current <= panels.size()) {
                int counter = 0;
                for (Iterator iter = panels.iterator(); iter.hasNext(); counter++) {
                    WizardPanel panel = (WizardPanel) iter.next();
                    panel.setObject(dco);
                    panel.setVisible(counter == current);
                }
            } else {
                current -= 1;
            }
            
            applyPanel();
        } catch (WizardException wzexp) {
            new MessageBox(wzexp.getMessage(), MessageBox._WARNING);
        }
    }
    
    private void back() {
        current -= 1;
        int counter = 0;
        
        if (current >= 0) {  
            for (Iterator iter = panels.iterator(); iter.hasNext(); counter++) {
                WizardPanel panel = (WizardPanel) iter.next();
                panel.setVisible(counter == current);
            }
        } else {
            current += 1;
        }
        
        applyPanel();
    }    
    
    private void close() {
        dco = null;
        dispose();
    }
    
    private void build(Collection panels) {
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        JPanel panelActions = new JPanel(false);      
        JPanel panelInformation = new JPanel(false);
        
        panelActions.setBackground(color);
        panelInformation.setBackground(color);
        
        panelActions.setLayout(Layout.getGBL());
        panelInformation.setLayout(Layout.getGBL());
        
        // Help panel
        panelInformation.setLayout(Layout.getGBL());
        
        textHelp.setBorder(null);
        JScrollPane scroller = new JScrollPane(textHelp);
        scroller.setBorder(null);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        textHelp.setEditable(false);
        textHelp.setBackground(color);
        
        panelInformation.add(textHelp, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));
        
        // Action panel
        JButton buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
        buttonNext.addActionListener(new NextAction());
        buttonBack.addActionListener(new BackAction());
        buttonFinish.addActionListener(new FinishAction());
        buttonCancel.addActionListener(new CancelAction());

        buttonBack.setEnabled(false);
        buttonFinish.setVisible(false);
        
        panelActions.add(buttonBack,    Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,  Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonNext,    Layout.getGBC(2, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonFinish,  Layout.getGBC(2, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));        
        
        // Main
        getContentPane().setLayout(Layout.getGBL());

        getContentPane().add(panelInformation,Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                       new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,Layout.getGBC(1, 3, 1, 2, 1.0, 1.0
                                      ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                       new Insets(5, 5, 5, 5), 0, 0));
        // the actual content panels
        int counter = 0;
        for (Iterator iter = panels.iterator(); iter.hasNext(); counter++) {
            JPanel panel = (JPanel) iter.next();
            panel.setBackground(color);
            
            panel.setMinimumSize(new Dimension(350,350));
            panel.setPreferredSize(new Dimension(350,350));
            
            panel.setVisible(counter == 0);
            getContentPane().add(panel,   Layout.getGBC(1, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        }
        
        setResizable(true);
        pack();
        
        applyPanel();
    }
    
    private class NextAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            next();
        }
    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            saveSettings();
            close();
        }
    }

    
    private class BackAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            back();
        }
    }
    
    private class FinishAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            finish();
        }
    }      
}
