/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.settings.DcSettings;

/**
 * Godfather for all Data Crow actions
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.6
 */
public abstract class DCAction implements ActionListener {

    protected static final int _SEARCHTAB = 0;
    protected static final int _INSERTTAB = 1;
    protected static final int _NOTETAB = 2;
    
    public int getCurrentTab() {
    	return DataCrow.mainFrame.getSelectedTab();
    }
    
    public DCModule getModule() {
        int module = DcSettings.getValueAsInt(Repository.settings.stModule);
        return DCModules.getModule(module);
    }
    
    public void changeModule(int moduleIndex) {
        int activeModule = getModule().getIndex();
        if (moduleIndex != activeModule) {
            DataCrow.mainFrame.changeModule(moduleIndex);
        }    
    }
        
    public abstract void actionPerformed(ActionEvent ae);
}
