/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.audiocd.AudioCDQuickViewPane;
import net.sf.dc.console.audiocd.ImportAudioCDDialog;
import net.sf.dc.console.audiocd.InsertAudioCDPanel;
import net.sf.dc.console.audiocd.SearchAudioCDPanel;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.QuickViewPane;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.reporting.ReportFieldDefinitions;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Audio CD module
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class AudioCDModule extends DCModule {
    
    private ImportAudioCDDialog importAudioCDDialog = null;
    private final DCMenuBar menuBar;
    
    /**
     * Initializes this module
     */
    public AudioCDModule() {
        super(DCModules._AUDIOCD,
              Resources.getText(DCLabels.lblModuleAudioCD),
              "audiocds",
              "aucd",
              " audiocds LEFT JOIN audiotracks ON (audiocds.ID = audiotracks.Album)",
              false,
              false,
              true);
        menuBar = new DCMenuBar(this);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }
    
    public String getObjectName() {
    	return Resources.getText(DCLabels.lblAudioCD);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(Resources.getText(DCLabels.lblTools));
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
										        Resources.getText(DCLabels.lblAudioCDImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuImport);
        
    	return menu;
    }
    
    public DCMenuBar getMenuBar() {
    	return menuBar;
    }
    
    public DCObject getDCObject() {
        return new AudioCD();
    }

    protected void initializeUI() {
        insertPanel = new InsertAudioCDPanel();
        searchPanel = new SearchAudioCDPanel();
    }
        
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsAudioCD;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stAudioCDRequiredFields;
    }
    
    public void startImport() {
        if (importAudioCDDialog == null) {
            importAudioCDDialog = new ImportAudioCDDialog(InsertAudioCDPanel.class, 0);
        }
        importAudioCDDialog.setVisible(true);
        importAudioCDDialog.startSearch();
    }
    
    public QuickViewPane getQuickView() {
        return new AudioCDQuickViewPane();
    }
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
											  String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(AudioCD._ID, false, 0);
        definitions.addDefinition(AudioCD._B_ARTIST, true, 0);
        definitions.addDefinition(AudioCD._A_TITLE, true, 0);
        definitions.addDefinition(AudioCD._C_DESCRIPTION, false, 0);
        definitions.addDefinition(AudioCD._D_YEAR, true, 0);
        definitions.addDefinition(AudioCD._E_GENRE, false, 0);
        definitions.addDefinition(AudioCD._F_PICTUREFRONT, true, 0);
        definitions.addDefinition(AudioCD._G_PICTUREBACK , false, 0);
        definitions.addDefinition(AudioCD._H_PICTURECD , false, 0);
        definitions.addDefinition(AudioCD._I_DISCID , false, 0);
        definitions.addDefinition(AudioCD._J_LOCATION , false, 1);
        definitions.addDefinition(AudioCD._K_RATING , true, 0);
        return definitions;
    }
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DCFieldDefinition[12];
        int counter = 0; 
        fields[AudioCD._ID] = new DCFieldDefinition(
                                        counter++,
                                        Resources.getText(DCLabels.lblID),
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[AudioCD._A_TITLE] = new DCFieldDefinition(
                                        counter++,
                                        Resources.getText(DCLabels.lblTitle),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[AudioCD._B_ARTIST] = new DCFieldDefinition(
                                        counter++,
                                        Resources.getText(DCLabels.lblArtist),
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Artist");
        fields[AudioCD._C_DESCRIPTION] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblDescription),
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");        
        fields[AudioCD._D_YEAR] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblYear),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Year");                                       
        fields[AudioCD._E_GENRE] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblGenre),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");
        fields[AudioCD._F_PICTUREFRONT] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblPictureFront),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");
        fields[AudioCD._G_PICTUREBACK] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblPictureBack),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");
        fields[AudioCD._H_PICTURECD] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblPictureCD),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");
        fields[AudioCD._I_DISCID] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblDiscId),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "DiscID");                                     
        fields[AudioCD._J_LOCATION] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblLocation),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");          
        fields[AudioCD._K_RATING] = new DCFieldDefinition(counter++,
                                        Resources.getText(DCLabels.lblRating),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");         
    }
    
    public DCObject[] getDefaultData() {
        return new DCObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }       
}
