/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.audiocd;

import javax.swing.ImageIcon;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.settings.DcSettings;

/**
 * The Audio CD Object
 * @see net.sf.dc.core.objects.DCObject
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class AudioCD extends DCObject {

    public static final int _A_TITLE = 1;
    public static final int _B_ARTIST = 2;
    public static final int _C_DESCRIPTION = 3;
    public static final int _D_YEAR = 4;
    public static final int _E_GENRE = 5;
    public static final int _F_PICTUREFRONT = 6;
    public static final int _G_PICTUREBACK = 7;
    public static final int _H_PICTURECD = 8;
    public static final int _I_DISCID = 9;
    public static final int _J_LOCATION = 10;
    public static final int _K_RATING = 11;

    public AudioCD() {
        super();
    }
    
    /**
     * Sets the module index / sets the ownership of this data crow object  
     */    
    protected void setModuleIndex() {
        module = DCModules._AUDIOCD;
    }
    
    /**
     * Returns the icon for this object
     */    
    public ImageIcon getIcon() {
        return null;    
    }
    
    /**
     * Initialize all user settings on this data crow object 
     */    
    protected void initializeSettings() {
        DCFieldDefinition[] fields = getFields();
        
        int[] requiredFields = DcSettings.getValueAsIntegerArray(
                                  Repository.settings.stAudioCDRequiredFields);
                                  
        for (int i = 0; i < requiredFields.length; i++) {
            fields[requiredFields[i]].setRequired(true);
        }
                                  
        int[] hiddenFields = DcSettings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsAudioCD);   

        for (int i = 0; i < hiddenFields.length; i++) {
            fields[hiddenFields[i]].setHidden(true);
        }
    }
    
    public boolean equals(AudioCD audioCD) {
    	int trackCount1 = getChildren().size();
        String title1 = getValue(AudioCD._A_TITLE).toString().toLowerCase();
        String artist1 = getValue(AudioCD._B_ARTIST).toString().toLowerCase();
        
        String title2 = audioCD.getValue(AudioCD._A_TITLE).toString().toLowerCase();
        String artist2 = audioCD.getValue(AudioCD._B_ARTIST).toString().toLowerCase();
        int trackCount2 = audioCD.getChildren().size();
        
        if (title1.equals(title2) && artist1.equals(artist2) &&
            trackCount1 == trackCount2) {
        	return true;
        } else {
        	return false;
        }
    }
    
    /**
     * Create a UI representation for this object
     */
    public String toString() {
        return getValue(AudioCD._A_TITLE).toString();
    } 
}
