package net.sf.dc.core.objects.reporting;

import net.sf.dc.core.modules.DCModule;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

public class ReportFieldDefinition {

    public final static int _LEFT = 0;
    public final static int _RIGHT = 1;
    public final static int _CENTER = 2;
    
    private int index = 0;
    private boolean enabled = true;
    private int alignment = 1;
    
    protected ReportFieldDefinition(int index, boolean enabled, int alignment) {
        this.index = index;
        this.enabled = enabled;
        this.alignment = alignment;
    }
    
    protected ReportFieldDefinition(int index, boolean enabled, String alignment) {
        this.index = index;
        this.enabled = enabled;
        this.alignment = getAlignmentIndex(alignment);
    }    
    
    public Object[] getDisplayValues(DCModule module) {
        return new Object[] {module.getFieldDefinition(index),
                             new Boolean(enabled),
                             getAlignmentLabel()};
    }
    
    private int getAlignmentIndex(String label) {
        if (label.equals(Resources.getText(DCLabels.lblLeft))) {
            return ReportFieldDefinition._LEFT;
        } else if (label.equals(Resources.getText(DCLabels.lblRight))) {
            return ReportFieldDefinition._RIGHT;
        } else {
            return ReportFieldDefinition._CENTER;
        }
    }
    
    private String getAlignmentLabel() {
        if (alignment == ReportFieldDefinition._LEFT) {
            return Resources.getText(DCLabels.lblLeft);
        } else if (alignment == ReportFieldDefinition._RIGHT) {
            return Resources.getText(DCLabels.lblRight);
        } else {
            return Resources.getText(DCLabels.lblCenter);   
        }
    }
        
    public int getIndex() {
        return index;
    }
    
    public int getAlignment() {
        return alignment;
    }
    
    public boolean isEnabled() {
        return enabled;
    }  
}
