/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.ReportTextPositionCombo;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.reporting.ReportFieldDefinition;
import net.sf.dc.core.objects.reporting.ReportFieldDefinitions;
import net.sf.dc.db.Query;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.db.QueryWhat;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.Resources;
import net.sf.ext.util.StringUtils;
import net.sf.ext.util.Utilities;

import com.lowagie.text.Cell;
import com.lowagie.text.Chapter;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;

/**
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.1
 */
public class CardReport extends Report {
    
    public CardReport(  ReportingDialog dialog, 
                        DCObject[] data, 
                        File target,
                        int module,
                        ReportFieldDefinitions definitions) {
        super();
        
        ReportSettings.setCurrentReport(ReportSettings._CARD_REPORT, module);
        
        this.dialog = dialog;
        this.data = data;
        this.target = target;
        this.definitions = definitions;
    }
    
    public void create() throws Exception {
        initialize();

        int textPosition = ReportSettings.getValueAsInt(Repository.reportsettings.stLabelPosition);
        int collabelspan = 1;
        int coltextspan = 3;
        if (textPosition == ReportTextPositionCombo._BELOWLABELS) {
            collabelspan = 4;
            coltextspan = 4;
        }            
        
        for (int j = 0; j < data.length && keepOnRunning; j++) {
            Table datatable = getTable(4);
            
            DCObject o = data[j];
            dialog.addMessage(Resources.getText(DCMessages.msgAddingToReport, o.toString()));
            
            Collection pictureFields = new ArrayList();
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
                
                if (definition.isEnabled()) {
                    int field = definition.getIndex();
                    
                	String s = o.getDisplayString(field);
                    int fieldType = o.getFieldType(field);
                	if (s.length() > 0 && !s.equals("0") && !s.equals("-1")) {
                        String label = o.getField(field).getLabel();
                        
                        Cell cellField = getLabelCell(label, definition);
                        cellField.setColspan(collabelspan);
                		datatable.addCell(cellField);
        
                        if (fieldType == ComponentFactory._RATINGCOMBOBOX) {
                            Object value = o.getValue(field);
                            Cell ratingCell = getRatingCell(value, definition);
                            ratingCell.setVerticalAlignment(Cell.ALIGN_BOTTOM);
                            ratingCell.setColspan(coltextspan);
                            datatable.addCell(ratingCell);
                        } else {
                    		Cell cellValue = getTextCell(s, definition);
                            cellValue.setColspan(coltextspan);
                    		datatable.addCell(cellValue);
                        }
                        
                	} else if (    fieldType == ComponentFactory._PICTUREFIELD && 
                                   o.getValue(field) != null) {
                        
                        s = (String) o.getValue(field);
                        if (!StringUtils.isEmpty(s)) {
                            ImageIcon icon = Utilities.base64ToImage(s);
                            pictureFields.add(getImageCell(icon, definition, 200, 150, false));
                        }
                    }
                } // enabled
            }
            
            Table pictureTable = getImageTable(pictureFields);
            
            Paragraph cTitle = new Paragraph(o.toString(), labelFont);
 
            Table childrenTable = getAudioTrackTable(o);
            boolean createIndex = ReportSettings.getValueAsBoolean(Repository.reportsettings.stCreateIndex);
            
            datatable.complete();
            if (!createIndex) {
                document.add(datatable);
                document.add(pictureTable);
                if (childrenTable != null) document.add(childrenTable);
            } else {
                Chapter chapter = new Chapter(cTitle, j + 1);
                chapter.add(datatable);    
                chapter.add(pictureTable);
                if (childrenTable != null) chapter.add(childrenTable);
                document.add(chapter);
            }
            
            document.newPage();   
            dialog.updateProgressBar(j + 1);
        }
        document.close();
        dialog.addMessage(Resources.getText(DCLabels.lblExportHasFinished));
    }
    
    private Table getImageTable(Collection cells) throws Exception {
        Table table = getTable(2);
        for (Iterator iter = cells.iterator(); iter.hasNext(); ) {
            table.addCell((Cell) iter.next());
        }     
        return table;
    }
    
    private Table getAudioTrackTable(DCObject parent) throws Exception {
        int module = DcSettings.getValueAsInt(Repository.settings.stModule);
        
        DCModule childModule = DCModules.getModule(module).getChildModule();
        
        Table childrenTable = null;
        if (childModule != null) {
            DCObject child = childModule.getDCObject();
            child.setValue(child.getParentReferenceFieldIndex(), parent.getID());
            QueryOptions options = new QueryOptions("2", true, true);
            QueryWhat queryWhat = new QueryWhat(child, child.getDatabaseFieldNames());
            Query query = new Query(queryWhat, child.toQueryCondition(true), options, Query._SELECT, null, null);
            
            Collection children = new WorkFlow().runQueryDirect(query);
            
            if (children != null && children.size() > 0) {
                int[] indices = new int[] {AudioTrack._A_TRACKNUMBER, AudioTrack._B_TITLE, AudioTrack._D_PLAYLENGTH};
                childrenTable = getTable(3);
                childrenTable.setWidths(new float[] {1,5,1});
    
                for (int i = 0; i < indices.length; i++) {
                    String label = child.getField(indices[i]).getLabel();
                    childrenTable.addCell(getLabelCell(label, null));
                }
                
                for (Iterator iter = children.iterator(); iter.hasNext(); ) {
                    DCObject o = (DCObject)  iter.next();
                    for (int j = 0; j < indices.length; j++) {
                        String s = o.getDisplayString(indices[j]);
                        childrenTable.addCell(getTextCell(s, null));
                    }
                }
            }
        }
        return childrenTable;
    }    
}
