/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.io.File;

import javax.swing.JFrame;

import net.sf.dc.console.components.ColorSelector;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.objects.book.BookCategory;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCTooltips;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.jflexsett.settings.ui.SettingsView;
import net.sf.ext.util.Resources;

/**
 * Groups are created here, as well as the settings. A similar class like this 
 * needs to be implemented in the project which uses jFlexSett.
 * 
 * All settings and their values (and the settings groups) can
 * be found by referencing to the Settings class. 
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class DcSettings {

    public static final String _General = 
                                Resources.getText(DCLabels.lblGroupGeneral);
    
    public static final String _RequiredBookFields = 
                                Resources.getText(DCLabels.lblBookRequiredFields);
    public static final String _RequiredSoftFields = 
                                Resources.getText(DCLabels.lblSoftwareRequiredFields);
    public static final String _RequiredMusicFields = 
                                Resources.getText(DCLabels.lblMusicRequiredFields);
    public static final String _RequiredMovieFields = 
                                Resources.getText(DCLabels.lblMovieRequiredFields);
    public static final String _RequiredAudioCDFields = 
                                Resources.getText(DCLabels.lblAudioCDRequiredFields);
    public static final String _RequiredAudioTrackFields = 
                                Resources.getText(DCLabels.lblAudioTrackRequiredFields);
    public static final String _BackGroundColor = 
                                Resources.getText(DCLabels.lblColor);
    public static final String _TableColors =
                                Resources.getText(DCLabels.lblTableColorSettings);
    public static final String _OddRowColor = 
                                Resources.getText(DCLabels.lblOddRowColor);
    public static final String _EvenRowColor = 
                                Resources.getText(DCLabels.lblEvenRowColor);
    public static final String _HeaderColor =  
                                Resources.getText(DCLabels.lblTableHeaderColor);
    public static final String _RowSelectionColor =  
                                Resources.getText(DCLabels.lblRowSelectionColor);
    public static final String _HTTP =
                                Resources.getText(DCLabels.lblHTTPSettings);
    public static final String _Font =
                                Resources.getText(DCLabels.lblFont);
    
    private static Settings settings = new Settings();
    
    
    /**
     * Starts the initialization of the Settings and starts the UI
     */
    public DcSettings() {
        createGroups();
        createSettings();
        settings.setSettingsFile(new File(Repository.files.flIniFile));
        SettingsFile.load(settings);
    }

    public static void save() {
        SettingsFile.save(settings);        
    }
    
    public static SettingsView getView(JFrame parent) {
        SettingsView view = new SettingsView(parent, "Data Crow Settings Panel", settings);
        view.setDisclaimer("icons_system" + File.separator + "logo.jpg");
        //SettingsView.setSize(Settings.getValueAsDimension(Repository.settings.stSettingsViewSize));
        view.setSize(new Dimension(875, 450));
        Image image = IconLibrary._icoSettings.getImage();
        view.setIconImage(image);
        image.flush(); 
        return view;
    }
    
    public static String[] getValueAsStringArray(String key) {
        return settings.getValueAsStringArray(key);
    }
    
    public static int[] getValueAsIntegerArray(String key) {
        return settings.getValueAsIntegerArray(key);
    }

    public static Font getValueAsFont(String key) {
        return settings.getValueAsFont(key);
    }    
    
    public static int getValueAsInt(String key) {
        return settings.getValueAsInt(key);
    }

    public static boolean getValueAsBoolean(String key) {
        return settings.getValueAsBoolean(key);
    }


    public static Color getValueAsColor(String key) {
        return settings.getValueAsColor(key);
    }
    
    public static Dimension getValueAsDimension(String key) {
        return settings.getValueAsDimension(key);
    }


    public static String getValueAsString(String key) {
        return settings.getValueAsString(key);
    }

    public static void setValue(String key, Object value) {
        settings.setValue(key, value);
    }

    public static void setStringAsValue(String key, String value) {
        settings.setStringAsValue(key, value);
    }    
    
    /**
     * Creates the settings group (with their hierarchy)
     */
    private void createGroups() {
        SettingsGroup generalGroup = new SettingsGroup(_General, "dc.Settings.GeneralSettings");
        SettingsGroup tableColorGroup = new SettingsGroup(_TableColors, "dc.Settings.TableColors");
        SettingsGroup backGround = new SettingsGroup(_BackGroundColor, "dc.Settings.BackGroundColor");
        SettingsGroup font = new SettingsGroup(_Font, "dc.Settings.Font");
        SettingsGroup http = new SettingsGroup(_HTTP, "dc.Settings.HttpSettings"); 
        
        generalGroup.addChild(new SettingsGroup(_RequiredSoftFields, "dc.Settings.RequiredFieldSettings"));
        generalGroup.addChild(new SettingsGroup(_RequiredMusicFields, "dc.Settings.RequiredFieldSettings"));
        generalGroup.addChild(new SettingsGroup(_RequiredMovieFields, "dc.Settings.RequiredFieldSettings"));
        generalGroup.addChild(new SettingsGroup(_RequiredAudioCDFields, "dc.Settings.RequiredFieldSettings"));
        generalGroup.addChild(new SettingsGroup(_RequiredAudioTrackFields, "dc.Settings.RequiredFieldSettings"));
        generalGroup.addChild(new SettingsGroup(_RequiredBookFields, "dc.Settings.RequiredFieldSettings"));
        
        tableColorGroup.addChild(new SettingsGroup(_OddRowColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_EvenRowColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_HeaderColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_RowSelectionColor, "dc.Settings.TableColors"));
        
        settings.addGroup(_General, generalGroup);
        settings.addGroup(_BackGroundColor, backGround);
        settings.addGroup(_HTTP, http);
        settings.addGroup(_TableColors, tableColorGroup);
        settings.addGroup(_Font, font);
    }
    
    /**
     * Creates the settings and adds them to the specified group.
     * If a self-created component needs to be added, make sure this
     * components implements the ISettingsComponent class!
     * Otherwise the component is not supported.. The getValue and
     * setValue must correspond to the DataType of the setting.
     */
    private void createSettings() {
        settings.addSetting(_BackGroundColor,                                    // Group key
                    new Setting(Repository.datatypes._COLOR,            // Data type
                                Repository.settings.stColor,            // Key
                                new Color(230, 236, 230),               // Default value
                                new ColorSelector(true, 
                                    Repository.settings.stColor),       // UI Component
                                "",                                     // Help text
                                Resources.getText(DCLabels.lblColor),   // Label text
                                false,                                  // show label ?
                                true));                                 // Add to panel ?
        
        settings.addSetting(_RowSelectionColor, 
                    new Setting(Repository.datatypes._COLOR, 
                                Repository.settings.stRowSelectionColor,
                                new Color(192, 197, 214), 
                                new ColorSelector(false,
                                    Repository.settings.stRowSelectionColor),
                                "", 
                                Resources.getText(DCLabels.lblRowSelectionColor), 
                                false, 
                                true));         
        
        settings.addSetting(_OddRowColor, 
                    new Setting(Repository.datatypes._COLOR, 
                    		    Repository.settings.stOddRowColor,
								new Color(224, 238, 227), 
                                new ColorSelector(false,
                                    Repository.settings.stOddRowColor),
							    "", 
                                Resources.getText(DCLabels.lblOddRowColor), 
                                false, 
                                true));

        settings.addSetting(_EvenRowColor, 
                    new Setting(Repository.datatypes._COLOR,
                    		    Repository.settings.stEvenRowColor, 
                                new Color(247, 247, 239), 
                                new ColorSelector(false,
                                    Repository.settings.stEvenRowColor),
                                "",
								Resources.getText(DCLabels.lblEvenRowColor), 
                                false, 
                                true));
        
        settings.addSetting(_HeaderColor, 
                    new Setting(Repository.datatypes._COLOR, 
                                Repository.settings.stTableHeaderColor,
                                new Color(211, 220, 221), 
                                new ColorSelector(false,
                                    Repository.settings.stTableHeaderColor),
                                "", 
                                Resources.getText(DCLabels.lblTableHeaderColor), 
                                false, 
                                true));         
        settings.addSetting(_General,
                    new Setting(  Repository.datatypes._INTEGER,
                                Repository.settings.stMaximumSearchRes,
                                new Integer(100),
                                ComponentFactory._NUMBERFIELD,
                                Resources.getText(DCTooltips.tpSearchRes),
                                Resources.getText(DCLabels.lblMaximumSearchRes),
                                true,
                                true));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCheckUniqueness,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpUniqueness),
                                Resources.getText(DCLabels.lblUniqueness),
                                false,
                                true));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stCheckRequiredFields,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpRequiredFields),
                                Resources.getText(DCLabels.lblRequired),
                                false,
                                true));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stConnectionString,
                                "dc",
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stDatabaseDriver,
                                "org.hsqldb.jdbcDriver",
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stShowTableTooltip,
                                new Boolean(true),
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpShowTableTooltips),
                                Resources.getText(DCLabels.lblShowTableTooltips),
                                false,
                                true));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.settings.stModule,
                                new Integer(1),
                                null,
                                "",
                                "",
                                true,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stAutoLoadData,
                                Boolean.TRUE,
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpAutoLoadData),
                                Resources.getText(DCLabels.lblAutoLoadData),
                                false,
                                true));         
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stShowHiddenFieldsInForm,
                                Boolean.TRUE,
                                ComponentFactory._CHECKBOX,
                                Resources.getText(DCTooltips.tpShowHiddenFieldsInForms),
                                Resources.getText(DCLabels.lblShowHiddenFieldsInForms),
                                false,
                                true));             
        settings.addSetting(_Font,
                    new Setting(Repository.datatypes._FONT,
                    		    Repository.settings.stSystemFontNormal,
								new Font("Arial", Font.PLAIN, 10),
								ComponentFactory._FONTSELECTOR,
								Resources.getText(DCTooltips.tpFont),
								Resources.getText(DCLabels.lblFontNormal),
								true,
								true));
        settings.addSetting(_Font,
                    new Setting(Repository.datatypes._FONT,
                    		    Repository.settings.stSystemFontBold,
                                new Font("Arial", Font.PLAIN, 10),
								ComponentFactory._FONTSELECTOR,
								Resources.getText(DCTooltips.tpFont),
								Resources.getText(DCLabels.lblFontBold),
								true,
								true));        
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stHTTPUserAgent,
                                "Mozilla/4.04 (Windows; I; Win95)",
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stProxyServerName,
                                "",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpProxyServerName),
                                Resources.getText(DCLabels.lblProxyServerName),
                                true,
                                true));
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.settings.stProxyServerPort,
                                new Integer(0),
                                ComponentFactory._NUMBERFIELD,
                                Resources.getText(DCTooltips.tpProxyServerPort),
                                Resources.getText(DCLabels.lblProxyServerPort),
                                true,
                                true));
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stProxyUserName,
                                "",
                                ComponentFactory._SHORTTEXTFIELD,
                                Resources.getText(DCTooltips.tpProxyUserName),
                                Resources.getText(DCLabels.lblProxyUserName),
                                true,
                                true));
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stProxyPassword,
                                "",
                                ComponentFactory._PASSWORDFIELD,
                                Resources.getText(DCTooltips.tpProxyPassword),
                                Resources.getText(DCLabels.lblProxyPassword),
                                true,
                                true));        
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stBrowserPath,
                                "",
                                ComponentFactory._FILEFIELD,
                                "",
                                Resources.getText(DCLabels.lblBrowserPath),
                                true,
                                true));
        settings.addSetting(_RequiredMusicFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stMusicRequiredFields,
                                new int[] {MusicFile._F_FILENAME},
                                ComponentFactory._MUSICREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));
        settings.addSetting(_RequiredMovieFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stMovieRequiredFields,
                                new int[] {Movie._A_TITLE},
								ComponentFactory._MOVIEREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));
        settings.addSetting(_RequiredAudioCDFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stAudioCDRequiredFields,
                                new int[] {AudioCD._A_TITLE},
                                ComponentFactory._AUDIOCDREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));        
        settings.addSetting(_RequiredAudioTrackFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stAudioTrackRequiredFields,
                                new int[] {AudioTrack._A_TRACKNUMBER},
                                ComponentFactory._AUDIOTRACKREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));          
        settings.addSetting(_RequiredSoftFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stSoftwareRequiredFields,
                                new int[] {Software._A_NAME},
                                ComponentFactory._SOFTWAREREQFIELDTABLE,
                                "",
                                "",
                                false,
                                true));
        settings.addSetting(_RequiredBookFields,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stBookRequiredFields,
                                new int[] {Book._A_TITLE},
                                ComponentFactory._BOOKREQUIREDFIELDTABLE,
                                "",
                                "",
                                false,
                                true));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stSoftwareCatRequiredFields,
                                new int[] {SoftwareCategory._A_NAME},
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stBookCatRequiredFields,
                                new int[] {BookCategory._A_NAME},
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSoftwareCatFormSize,
                                new Dimension(700, 600),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stBookCatFormSize,
                                new Dimension(700, 600),
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stExpertFormSize,
                                new Dimension(700, 600),
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stHelpFormSize,
                                new Dimension(700, 600),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stTabbedPaneSize,
                                new Dimension(741, 496),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSoftwareItemFormViewSize,
                                new Dimension(450, 550),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stBookItemFormViewSize,
                                new Dimension(450, 550),
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMusicItemFormViewSize,
                                new Dimension(450, 550),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMovieItemFormViewSize,
                                new Dimension(450, 550),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stWizardFormSize,
                                new Dimension(450, 550),
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stItemFormViewSize,
                                new Dimension(450, 550),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stSoftwareCatItemFormViewSize,
                                new Dimension(407, 165),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stBookCatItemFormViewSize,
                                new Dimension(407, 165),
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMusicOrganizerViewSize,
                                new Dimension(300, 400),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stUpdateAllDialogSize,
                                new Dimension(600, 300),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stMainViewSize,
                                new Dimension(865, 593),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stFreedbServer,
                                "freedb.freedb.org",
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_HTTP,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stAmazonServer,
                                "http://xml.amazon.com",
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stFieldSelectorSize,
                                new Dimension(500, 300),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stTextViewerSize,
                                new Dimension(500, 700),
                                null,
                                "",
                                "",
                                false,
                                false));
         settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stImdbSearchFormViewSize,
                                new Dimension(547, 469),
                                null,
                                "",
                                "",
                                false,
                                false));
         settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stAmazonSearchFormViewSize,
                                new Dimension(533, 495),
                                null,
                                "",
                                "",
                                false,
                                false));
         settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                             Repository.settings.stReportingDialogSize,
                             new Dimension(533, 490),
                             null,
                             "",
                             "",
                             false,
                             false));          
         
         settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stFreeDBSearchFormViewSize,
                                new Dimension(533, 495),
                                null,
                                "",
                                "",
                                false,
                                false));         
         settings.addSetting(_General,
                    new Setting(Repository.datatypes._DIMENSION,
                                Repository.settings.stImportAudioCDDialogSize,
                                new Dimension(542, 680),
                                null,
                                "",
                                "",
                                false,
                                false));         
        settings.addSetting(_General,
                   new Setting(Repository.datatypes._DIMENSION,
                               Repository.settings.stImportCDDialogSize,
                               new Dimension(550, 400),
                               null,
                               "",
                               "",
                               false,
                               false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stShowModuleList,
                                new Boolean(true),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stODBOverwriteValues,
                                new Boolean(false),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stBroadband,
                                new Boolean(true),
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsSoftware,
                                new int[] { Software._ID,
                                            Software._B_DESCRIPTION,
                                            Software._C_DEVELOPER,
                                            Software._D_PUBLISHER,
                                            Software._E_PLATFORM,
                                            Software._F_RELEASEYEAR,
                                            Software._G_WEBPAGE, 
                                            Software._H_FILENAME,
                                            Software._J_MULTI,
                                            Software._M_PICTURECD, 
                                            Software._O_SCREENSHOTTWO, 
                                            Software._L_PICTUREBACK,
                                            Software._P_SCREENSHOTTHREE,
                                            Software._Q_SERIALKEY,
                                            Software._R_FILESIZE},
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsBook,
                                new int[] { Book._ID, 
                                            Book._B_DESCRIPTION,
                                            Book._C_PUBLISHER,
                                            Book._E_RELEASEYEAR,
                                            Book._F_WEBPAGE,
                                            Book._I_PICTUREFRONT,
                                            Book._H_ISBN},
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsAudioCD,
                                new int[] {AudioCD._ID, 
                                           AudioCD._C_DESCRIPTION,
                                           AudioCD._D_YEAR,
                                           AudioCD._F_PICTUREFRONT,
                                           AudioCD._G_PICTUREBACK,
                                           AudioCD._H_PICTURECD,
                                           AudioCD._I_DISCID},
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsAudioTrack,
                                new int[] {AudioTrack._ID, AudioTrack._E_ALBUM},
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsMovie,
                                new int[] {Movie._ID,
                                           Movie._0_SUBTITLELANGUAGE,
                                           Movie._1_CERTIFICATION,
                                           Movie._2_FILESIZE,
                                           Movie._B_TITLE_LOCAL,
                                           Movie._C_DESCRIPTION,
                                           Movie._D_WEBPAGE,
                                           Movie._F_ACTORS,
                                           Movie._G_DIRECTOR,
                                           Movie._H_COUNTRY,
                                           Movie._I_YEAR,
                                           Movie._J_PLAYLENGTH,
                                           Movie._K_FILENAME,
                                           Movie._L_VIDEOCODEC,
                                           Movie._M_AUDIOCODEC,
                                           Movie._N_WIDTH,
                                           Movie._O_HEIGHT,
                                           Movie._P_FPS,
                                           Movie._Q_FRAMES,
                                           Movie._R_AUDIOBITRATE,
                                           Movie._S_AUDIOSAMPLINGRATE,
                                           Movie._T_AUDIOCHANNEL,
                                           Movie._U_VIDEOBITRATE,
                                           Movie._V_PICTUREFRONT,
                                           Movie._W_PICTUREBACK,
                                           Movie._X_PICTURECD,
                                           Movie._Z_AUDIOLANGUAGE},
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsMusic,
                                new int[] {MusicFile._ID,
                                           MusicFile._F_FILENAME,
                                           MusicFile._D_YEAR,
                                           MusicFile._H_QUALITY,
                                           MusicFile._I_ENCODING,
                                           MusicFile._E_GENRE,
                                           MusicFile._L_LOCATION,
                                           MusicFile._J_LYRIC,
                                           MusicFile._K_FILESIZE},
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsSoftwareCat,
                                new int[] {SoftwareCategory._ID},
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._INTEGERARRAY,
                                Repository.settings.stHiddenFieldsBookCat,
                                new int[] {BookCategory._ID},
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stLastDirectoryUsed,
                                "",
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._STRING,
                                Repository.settings.stImdbServer,
                                "http://www.imdb.com",
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stShowMenuBarLabels,
                                Boolean.TRUE,
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stUpdateAllSelectedItemsOnly,
                                Boolean.TRUE,
                                null,
                                "",
                                "",
                                false,
                                false));
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stShowQuickView,
                                Boolean.TRUE,
                                null,
                                "",
                                "",
                                false,
                                false));        
        settings.addSetting(_General,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.settings.stDeleteImageFileAfterImport,
                                Boolean.FALSE,
                                ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblDeleteImageFile)),
                                Resources.getText(DCTooltips.tpDeleteImageFile),
                                "",
                                false,
                                true));          
    }
}
