/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.wf.requests;

import java.util.Collection;

import net.sf.dc.console.MainFrame;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;

/**
 * Request for updating a row in the table
 * 
 * @author Robert Jan van der Waals
 * @version 1.6
 * @since 1.4
 */
public class UpdateRowRequest implements IRequest {

    private final boolean overwrite;
    private final boolean allowDeletes;
    private final int tab;
    private final DCObject dco;
    private boolean executeOnFail = false;
    
    public UpdateRowRequest (int tab, DCObject dco, boolean overwrite, boolean allowDeletes) {
        this.overwrite = overwrite;
        this.allowDeletes = allowDeletes;
        this.tab = tab;
        this.dco = dco;
    }
    
    public void execute(Collection objects) {
        DCModule mod = dco.getModule();
        if (tab == MainFrame._INSERTTAB) {
        	mod.getInsertPanel().updateRow(dco.getID(), dco, overwrite, allowDeletes);
        } else if (tab == MainFrame._SEARCHTAB) {
            mod.getSearchPanel().updateRow(dco.getID(), dco, overwrite, allowDeletes);
        }
    }
    
    public boolean getExecuteOnFail() {
        return executeOnFail;
    }
    
    public void setExecuteOnFail(boolean b) {
        executeOnFail = b;
    }        
}
